///	@file	strutil.cpp
///	@brief	strutil.cpp
//#include <sstream>
#include <iomanip>
#include <math.h>
#include <ctype.h>

#include "strutil.h"
#include "mlslocale.h"

using namespace std;
using namespace MLS;

///	@brief	글를 소문자로 만들다.
///	@return	소문자로 만들어질 글
string strutil::tolower(
	const string &p			///< 소문자로 만들 글
	)
{
	string r = p;
	std::transform(r.begin(), r.end(), r.begin(), (int(*)(int)) std::tolower);
	return r;
}

///	@brief	글를 대문자로 만들다.
///	@return	대문자로 만들어질 글
string strutil::toupper(
	const string &p			///< 대문자로 만들 글
	)
{
	string r = p;
	std::transform(r.begin(), r.end(), r.begin(), (int(*)(int)) std::toupper);
	return r;
}

/// @brief	필요 없는 문자를 제거한다.
///	@return	공백, 엔터등을 제거한 문자열 알맹이
string strutil::chop(
	const string &p			///< 문자열 
	)
{
	string::size_type f = p.find_first_not_of(" \n\r");
	string::size_type r = p.find_last_not_of(" \n\r");
	
	if (f == string::npos || r== string::npos) return "";
	
	return p.substr(f, r-f+1);
}

/// @brief	p문자열에서 위치 f, r 사이의 문자열을 반환한다.
string strutil::getbetween(
	const string &p,			///< 문자열
	const char f,				///< 반환할 문자의 시작 위치
	const char r				///< 반환할 문자의 끝위치
	)
{
	string::size_type s = p.find_first_of(f);
	if (s == string::npos) return "";
	string::size_type s2 = p.find_first_of(r, s+1);
	if (s2 == string::npos) return "";	
	
	return p.substr(s+1 , s2-s-1);	
}

///	@brief	해당되는 글자를 다른 글자로 바꾼다.
///	@return	바꾸어진 글
string strutil::replace(
	const string &p,		///< 글자를 바꿀 대상 전체
	const char*find,		///< 바꿀 대상 문자
	const char*replace		///< 바꿀 문자
	)
{
	//  01234567890123456789012
	// "the great great bassist" "great" "brother"	
	string arg;
	
	string::size_type t = 0;
	string::size_type t2= p.find(find);
	
	string::size_type len = strlen(find);
	
	while(t2 != string::npos)
	{
		arg += p.substr(t, t2-t);
		arg += replace;
		
		t = t2 + len;
		t2 = p.find(find, t);
	}
	
	arg += p.substr(t);
	
	return arg;
}

///	@brief	숫자를 문자로 바꾸어준다.
///	@return	문자화 된 숫자
string strutil::toregular(
	unsigned int number		///< number	문자로 바꿀 숫자
	)
{
	unsigned int n=1, p=1;
	string str;	
	str.reserve(10);
			
	while(number/10 >= p) {p*=10; n+=1;}
	
	while (n)
	{
		str += '0' + number / p;
		
		number %= p;
		p /= 10;
		n--;
		
		if (n!=0 && n%3==0) str += ',';	
	}
	
	return str;
}

/// @brief	string에 slash를 추가하는 함수
///
/// 파일이름중에서 특수문자가(예 공백이나 () & *등..을 '\ ' '\ ('로 바꾸어
/// 주는함수
///
/// @return 	shlash가 추가된 글.
string strutil::addslash(
	const string &p			///< 문자열.
	)
{
	//  t
	// 01234
	// jones
	string arg;
	
	string::size_type t  = 0,
					  t2 = p.find_first_of(" ()&*?$'\"\\\0");
	
	while(t2 != string::npos)
	{
		arg.append(p, t, t2-t);
		arg += '\\';
		arg += p[t2];		
		t = t2 + 1;
		t2 = p.find_first_of(" ()&*?$'\"\\\0", t);
	}	
	arg.append(p, t, p.length()-t);
	
	return arg;			
}

/// @brief  char 를 wchar_t 형태로 바꾼다. setlocale(LC_TYPE, locale)을 사용해야 함.
wstring strutil::strtowstr(const string& str)
{
	string 	s = str;
	if (s.size() == 0 && s.size() > 10000000) return L"";
	size_t 		nSize;
	wstring		wstr;
	
	wchar_t*	w = new wchar_t[s.size()+1];
	memset(w, 0, (s.size()+1)*sizeof(wchar_t));
	
	if (nSize = mbstowcs(w, s.c_str(), s.size()) == -1)
	{
		LOG("mbstowcs func err [%s] [%d] [%d] [%s]", s.c_str(), errno, strerror(errno));
		delete w;
		return L"";
	}
	wstr = w;
	delete w;
	return wstr;
}

/// @brief  wchar_t를 char 형태로 바꾼다. setlocale(LC_TYPE, locale)을 사용해야 함.
string	strutil::wstrtostr(const wstring& wstr)
{
	wstring	s = wstr;
	if (s.size() == 0) return "";
	int 	nSize = 0;
	string  str;
	char	mbchar[10];
	int		nChar;

	for (int n = 0; n < s.size(); n++)
	{
		memset(&mbchar, 0, sizeof(mbchar));
		if (wctomb((char*)&mbchar, s[n]) == -1)
		{
			LOG("wcstombs func err [%d] num[%d] [%d] [%d] [%s]", (int)s[n], n, errno, strerror(errno));
		}
		str.append(mbchar);
	}
	return str;
}

/// @brief  화면에 맞게 스트링사이즈를 구한다. setlocale(LC_TYPE, locale)을 사용해야 함.
int strutil::krstrlen(
	const string& sSrc
	)
{
	if (sSrc.size() == 0) return 0;
	
	int		nLength = 0, nWidth = 0;
	string	s = sSrc;
	wstring wc = strtowstr(s);

	if (wc.size() == 0) return 0;

	for (int n = 0; n < wc.size(); n++)
	{
		if (wc[n] >= 0x100)
		{
			if ((nWidth = wcwidth(wc[n])) == -1)
			{
				LOG("krstrlen :: wcwidth func err [%d] ['%s'::%d] [%d] [%d] [%s]", n, sSrc.c_str(), sSrc.size(), errno, strerror(errno));
				nWidth = 0;
				continue;
			}
			nLength = nLength + nWidth;
		}
		else
		{
			nLength = nLength + 1;
		}
		
	}
	return nLength;
}

/// @brief  화면에 맞는 스트링사이즈를 구한다. setlocale(LC_TYPE, locale)을 사용해야 함.
int	strutil::wstrlen(const wstring& w)
{
	int	nWidth = 0, nLength = 0;
	wstring wstr = w;
	if (wstr.size() == 0) return 0;
	
	for (int n = 0; n < wstr.size(); n++)
	{
		if (wstr[n] >= 0x100)
		{
			if ((nWidth = wcwidth(wstr[n])) == -1) {
				LOG("wstrlen err [%d] [%d]", (int)wstr[n], n);
				nWidth = 0;
				continue;
			}
			nLength = nLength + nWidth;
		}
		else
		{
			nLength = nLength + 1;
		}
		
	}
	return nLength;
}

/// @brief  문자 길이에 맞게 자를때 씀. setlocale(LC_TYPE, locale)을 사용해야 함.
string strutil::krstrncpy(
	const string &sSrc,			///< 한글이나 영문자.
	int nPoint,					///< 위치
	int nLength					///< 문자열 길이.
	)
{
	string	sRt = "";
	if (sSrc.size() == 0) return sRt;
	string	s = sSrc;
	wstring wsrc = strtowstr(s);

	if (wsrc.size() == 0) return sRt;

	char	mbchar[10];
	int nPos = 0, nLen = 0, nWidth = 0;

	for (int n = 0; n < wsrc.size(); n++)
	{
		if (wsrc[n] >= 0x100)
		{
			if ((nWidth = wcwidth(wsrc[n])) == -1)
			{
				LOG("krstrncpy wcwidth func err [%d] [%s] [%d] [%d] [%s]", n, sSrc.c_str(), errno, strerror(errno));
				nWidth = 1;
				continue;
			}
		}
		else
		{
			nWidth = 1;
		}

		if (nLength <= nLen) break;
		
		if (nPos >= nPoint)
		{
			memset(&mbchar, 0, sizeof(mbchar));
			if (wctomb((char*)&mbchar, wsrc[n]) == -1)
			{
				LOG("krstrncpy wcstombs func err [%d] num[%d] [%d] [%d] [%s]", (int)wsrc[n], n, errno, strerror(errno));
			}
			else
			{
				sRt.append(mbchar);
			}
			nLen = nLen + nWidth;
		}

		nPos = nPos + nWidth;
	}
	return sRt;
}

/// @brief nLength 사이즈에 맞게 문자가 중간에 위치하게 만든다.
string strutil::strmid(
	int nLength,
	const string &sSrc
	)
{
	string sConvert;
	
	if (nLength == 0) return "";
	if (sSrc.size() == 0) return "";
	int nSrcStrLength = strutil::krstrlen(sSrc);
	if (nSrcStrLength >= nLength) return sSrc;	
	int nPos = (nLength - nSrcStrLength) / 2;
	sConvert.append(nPos, ' ');
	sConvert.append(sSrc);
	sConvert.append(nLength-(nPos+nSrcStrLength), ' ');
	return sConvert;
}

